//==================================================================
//
//	File name	:	calendar.js
//
//	Summary		:	Calendar class
//
//	Ceated		:	2018/12/03
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (calendar) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	calendar.DEFAULT_FORMAT = "DD/MM/YYYY";
	calendar.SHORT_MONTH = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
	calendar.LONG_MONTH = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
	calendar.WEEK = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]
	calendar.MONTH_FORMAT = "MONTH, YYYY";
	
	// Private variables
	let oPopUp = null;
	let iaLastDay = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	
	
	//==========================================================
	//
	//	Class		:	Calendar
	//
	//	Summary		:	Class of calendar
	//
	//==========================================================
	calendar.Calendar = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// * ID[string]
					oRect,			// * Size and position[FixedRect]
					oTarget,		// Target textbox[TextBox]
					bEnabled,		// * Enabled or not[boolean]
					sFormat,		// Format of output [String]
					oDate,			// Initial date[Date]
					sTooltip,		// Tool tip text[String]
					iTabIndex,		// Tab index[int]
					fpChange)		// Function on change[function]
		{
			// Base class
			super(false, sID, false, oRect, bEnabled, iTabIndex);
			
			// Check included files
			if(!jaswin.button)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "button.js"),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
			
			if(oTarget)
			{
				if(!jaswin.textbox)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_INCLUDE, "textbox.js"),
												"calendar.js", "jaswin.calendar.Calendar.constructor");
			}
			
			// Check parameters
			if(oTarget)
			{
				if((typeof oTarget != "object")
				|| (!oTarget instanceof jaswin.textbox.TextBox))
				{
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_TEXTBOX, sID),
													"calendar.js", "jaswin.calendar.Calendar.constructor");
				}
			}
			
			if(sFormat)
			{
				if(typeof sFormat != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_FORMAT, sID),
												"calendar.js", "jaswin.calendar.Calendar.constructor");
			}
			
			if(oDate)
			{
				if((typeof oDate != "object")
				|| (!oDate instanceof Date))
				{
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_DATE, sID),
												"calendar.js", "jaswin.calendar.Calendar.constructor");
				}
			}
			
			if(sTooltip)
			{
				if(typeof sTooltip != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
												"calendar.js", "jaswin.calendar.Calendar.constructor");
			}
			
			if(fpChange)
			{
				if(typeof fpChange != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
												"calendar.js", "jaswin.button.Calendar.constructor");
			}
			
			// Set properties
			this.sheet = null;				// Calendar sheet element[DOM Element]
			this.prevMonth = null;			// Button for previous month[Button]
			this.nextMonth = null;			// Button for next month[Button]
			this.monthLabel = null;			// Label for current month[Label]
			this.container = null;			// Container for client area[DOM Element]
			this.target = oTarget;			// Target textbox[TextBox]
			this.onChange = fpChange;		// Function on change[function(Argument:Calendar class)]
			this.currentYear = null;		// Current year
			this.currentMonth = null;		// Current month
			
			if(sFormat)
				this.format = sFormat;
			else
				this.format = calendar.DEFAULT_FORMAT;
			
			if(oDate)
				this.date = oDate;
			else
				this.date = new Date();
			this.tempDate = new Date(this.date.getTime());
			
			// Create element
			this.element = document.createElement("div");
			
			if(this.enabled)
				this.element.className = "ico-calendar";
			else
				this.element.className = "ico-calendar-disabled";
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			
			this.focusElement = this.element;
			
			// Tooltip
			if((sTooltip) && (sTooltip.length > 0))
				this.element.title = sTooltip;
			
			// Tab index
			if(iTabIndex)
			{
				if(iTabIndex > 0)
				{
					this.tabIndex = iTabIndex;
					this.focusElement.tabIndex = this.tabIndex;
				}
			}
			
			// Calendar sheet
			this.sheet = document.createElement("div");
			this.sheet.className = "calendar";
			this.sheet.style.display = "none";
			this.element.appendChild(this.sheet);
			
			let oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
													2, 2, 16, 16);
			this.prevMonth = new jaswin.button.Button("Btn" + this.id + "_PrevMonth", oButtonRect,
														jaswin.button.STYLE_FLAT | jaswin.button.STYLE_ICON,
														true,
														jaswin.ICON_TRIANGLE_LEFT, null,
														0, clickPrevMonth);
			this.sheet.appendChild(this.prevMonth.element);
			
			oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
												2, 2, 16, 16);
			this.nextMonth = new jaswin.button.Button("Btn" + this.id + "_NextMonth", oButtonRect,
														jaswin.button.STYLE_FLAT | jaswin.button.STYLE_ICON,
														true,
														jaswin.ICON_TRIANGLE_RIGHT, null,
														0, clickNextMonth);
			this.sheet.appendChild(this.nextMonth.element);
			
			this.monthLabel = document.createElement("span");
			this.monthLabel.className = "calendar-month";
			this.sheet.appendChild(this.monthLabel);
			
			let oInner;
			
			oInner = document.createElement("div");
			oInner.className = "calendar-inner";
			this.sheet.appendChild(oInner);
			
			this.container = document.createElement("div");
			this.container.className = "calendar-container";
			oInner.appendChild(this.container);
			
			let iCounter;
			let oDiv;
			
			oDiv = document.createElement("div");
			oDiv.className = "calendar-sun";
			oDiv.innerHTML = calendar.WEEK[0];
			oInner.appendChild(oDiv);
			for(iCounter = 1; iCounter < 6; iCounter++)
			{
				oDiv = document.createElement("div");
				oDiv.className = "calendar-week";
				oDiv.innerHTML = calendar.WEEK[iCounter];
				oInner.appendChild(oDiv);
			}
			oDiv = document.createElement("div");
			oDiv.className = "calendar-sat";
			oDiv.innerHTML = calendar.WEEK[6];
			oInner.appendChild(oDiv);
			
			oDiv = document.createElement("div");
			oDiv.style.clear = "both";
			oInner.appendChild(oDiv);
			
			this.setMonth(this.date.getFullYear(), this.date.getMonth());
			
			// Event binding
			if(this.enabled)
			{
				this.element.addEventListener("click", this.clickEvent = (e) => {
					let oElement = e.target;
					if(!this.sheet.contains(oElement))
						this.clickCalendar();
				}, false);
				this.element.addEventListener("keydown", this.keyDownEvent = (e) => {
					this.keyDown(e);
				}, false);
			}
		}
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent(oParent)
		{
			super.setParent(oParent);
			
			let oParentWnd = jaswin.findParentWindow(this);
			
			if(oParentWnd != global)
				this.sheet.style.zIndex = 50 + jaswin.frame.windowCount + oParentWnd.modalLevel;
			
			this.prevMonth.setParent(this);
			this.nextMonth.setParent(this);
		}
		
		//==================================================
		//	Enable this calendar
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if(this.enabled)
			{
				this.element.className = "ico-calendar";
				if(!this.clickEvent)
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						let oElement = e.target;
						if(!this.sheet.contains(oElement))
							this.clickCalendar();
					}, false);
				}
				if(!this.keyDownEvent)
				{
					this.element.addEventListener("keydown", this.keyDownEvent = (e) => {
						this.keyDown(e);
					}, false);
				}
			}
		}
		
		//==================================================
		//	Disable this calendar
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			this.sheet.style.display = "none";
			oPopUp = null;
			
			this.element.className = "ico-calendar-disabled";
			if(this.clickEvent)
			{
				this.element.removeEventListener("click", this.clickEvent, false);
				this.clickEvent = null;
			}
			if(this.keyDownEvent)
			{
				this.element.removeEventListener("keydown", this.keyDownEvent, false);
				this.keyDownEvent = null;
			}
		}
		
		//==================================================
		//	Click this calendar
		//==================================================
		clickCalendar()
		{
			if(this.sheet.style.display == "none")
			{
				let oRect = this.element.getBoundingClientRect();
				let iLeft;
				let iTop;
				
				if(oRect.left + 16 - 160 - 2 < 0)
					iLeft = oRect.left * (-1);
				else
					iLeft = 16 - 160 - 2;
				this.sheet.style.left = iLeft + "px";
				
				if(oRect.top + 16 + 164 + 2 > window.innerHeight)
					iTop = -164 - 8;
				else
					iTop = 16;
				this.sheet.style.top = iTop + "px";
				
				this.tempDate = new Date(this.date.getTime());
				this.setMonth(this.date.getFullYear(), this.date.getMonth());
				
				this.element.className = "ico-calendar-on";
				this.sheet.style.display = "block";
				oPopUp = this;
				
				global.element.addEventListener("click", calendarCancel, false);
			}
			else
			{
				this.element.className = "ico-calendar";
				this.sheet.style.display = "none";
				oPopUp = null;
				global.element.removeEventListener("click", calendarCancel, false);
			}
		}
		
		//==================================================
		//	Set Month
		//==================================================
		setMonth(iYear, iMonth)
		{
			if(typeof iYear != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_YEAR, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			if(typeof iMonth != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MONTH, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			if(iMonth < 0 || iMonth > 11)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_MONTH, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			
			let sMonthStr;
			let iLastDay;
			let oDate;
			let iCounter;
			let oDiv;
			
			this.currentYear = iYear;
			this.currentMonth = iMonth;
			
			sMonthStr = calendar.MONTH_FORMAT.replace("YYYY", "" + iYear);
			sMonthStr = sMonthStr.replace("MONTH", calendar.LONG_MONTH[iMonth]);
			this.monthLabel.innerHTML = sMonthStr;
			
			if(iMonth == 1)
			{
				if(iYear % 400 == 0 || (iYear % 100 != 0 && iYear % 4 == 0))
					iLastDay = 29;
				else
					iLastDay = 28;
			}
			else
				iLastDay = iaLastDay[iMonth];
			
			this.container.innerHTML = "";
			oDate = new Date(iYear, iMonth, 1);
			for(iCounter = 0; iCounter < oDate.getDay(); iCounter++)
			{
				oDiv = document.createElement("div");
				oDiv.className = "calendar-blank";
				this.container.appendChild(oDiv);
			}
			for(iCounter = 1; iCounter <= iLastDay; iCounter++)
			{
				oDiv = document.createElement("div");
				oDiv.innerHTML = "" + iCounter;
				
				if((this.date.getFullYear() == this.currentYear)
				&& (this.date.getMonth() == this.currentMonth)
				&& (this.date.getDate() == iCounter))
					oDiv.className = "calendar-selected";
				else if((this.tempDate.getFullYear() == this.currentYear)
				&& (this.tempDate.getMonth() == this.currentMonth)
				&& (this.tempDate.getDate() == iCounter))
					oDiv.className = "calendar-focus";
				else
					oDiv.className = "calendar-day";
				this.container.appendChild(oDiv);
				oDiv.addEventListener("click", (e) => {
					let oElement = e.target;
					this.selectDay(parseInt(oElement.innerHTML));
				}, false);
			}
			
			oDiv = document.createElement("div");
			oDiv.style.clear = "both";
			this.container.appendChild(oDiv);
		}
		
		//==================================================
		//	Set Day
		//==================================================
		selectDay(iDay)
		{
			if(typeof iDay != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.selectDay");
			if(!this.container.childNodes.length)
				return;
			if(iDay < 1)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_DATE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.selectDay");
			let iCounter;
			let bFound = false;
			
			for(iCounter = 0; iCounter < this.container.childNodes.length - 1; iCounter++)
			{
				if(this.container.childNodes[iCounter].innerHTML == "" + iDay)
				{
					this.date = new Date(this.currentYear, this.currentMonth, iDay);
					this.container.childNodes[iCounter].className = "calendar-selected";
					this.tempDate = new Date(this.date.getTime());
					bFound = true;
				}
				else
				{
					if(this.container.childNodes[iCounter].className != "calendar-blank")
						this.container.childNodes[iCounter].className = "calendar-day";
				}
			}
			
			this.element.className = "ico-calendar";
			this.sheet.style.display = "none";
			oPopUp = null;
			global.element.removeEventListener("click", calendarCancel, false);
			
			if(!bFound)
				return;
			
			let sOutput = this.format.replace("YYYY", "" + this.currentYear);
			let sTmp;
			
			if(this.currentMonth + 1 < 10)
				sTmp = "0" + (this.currentMonth + 1);
			else
				sTmp = "" + (this.currentMonth + 1);
			
			sOutput = sOutput.replace("MM", sTmp);
			
			if(iDay < 10)
				sTmp = "0" + iDay;
			else
				sTmp = "" + iDay;
			sOutput = sOutput.replace("DD", sTmp);
			
			if(this.target)
				this.target.setValue(sOutput);
			if(this.onChange)
				this.onChange(this);
		}
		
		//==================================================
		//	Get selected value
		//==================================================
		getValue()
		{
			let sOutput = this.format.replace("YYYY", "" + this.currentYear);
			let sTmp;
			let iDay = this.date.getDate();
			
			if(this.currentMonth + 1 < 10)
				sTmp = "0" + (this.currentMonth + 1);
			else
				sTmp = "" + (this.currentMonth + 1);
			
			sOutput = sOutput.replace("MM", sTmp);
			
			if(iDay < 10)
				sTmp = "0" + iDay;
			else
				sTmp = "" + iDay;
			sOutput = sOutput.replace("DD", sTmp);
			
			return sOutput;
		}
		
		//==================================================
		//	Key down on this calendar
		//==================================================
		keyDown(e)
		{
			if((e.keyCode == 13) || (e.keyCode == 32))
			{
				// Enter or Space
				if(this.sheet.style.display == "none")
					this.clickCalendar();
				else
					this.selectDay(this.tempDate.getDate());
				
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 27)
			{
				// Esc
				if(this.sheet.style.display == "block")
					this.clickCalendar();
			}
			else
			{
				if((e.keyCode < 37) || (e.keyCode > 40))
					return true;
				
				if((this.tempDate.getFullYear() != this.currentYear)
				|| (this.tempDate.getMonth() != this.currentMonth))
					return true;
				
				if(e.keyCode == 37)
				{
					// Left
					this.tempDate.setDate(this.tempDate.getDate() - 1);
				}
				else if(e.keyCode == 39)
				{
					// Right
					this.tempDate.setDate(this.tempDate.getDate() + 1);
				}
				else if(e.keyCode == 38)
				{
					// Up
					this.tempDate.setDate(this.tempDate.getDate() - 7);
				}
				else if(e.keyCode == 40)
				{
					// Down
					this.tempDate.setDate(this.tempDate.getDate() + 7);
				}
				this.setMonth(this.tempDate.getFullYear(), this.tempDate.getMonth());
				
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			return true;
		}
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setTooltip");
			this.element.title = sTooltip;
		}
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent(fpChange)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setChangeEvent");
			this.onClick = fpClick;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	calendarCancel
	//
	//	Summary		:	Event handler of cancel a calendar
	//
	//==========================================================
	function calendarCancel(e)
	{
		if(!oPopUp)
			return;
		
		let oElement = e.target;
		
		if(!oPopUp.element.contains(oElement))
		{
			oPopUp.clickCalendar();
			return;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	clickPrevMonth
	//
	//	Summary		:	Event handler of click a previous month
	//
	//==========================================================
	function clickPrevMonth(oButton)
	{
		let oCalendar = null;
		let iYear;
		let iMonth;
		
		oCalendar = oButton.parent;
		if(oCalendar.currentMonth == 0)
		{
			iYear = oCalendar.currentYear - 1;
			iMonth = 11;
		}
		else
		{
			iYear = oCalendar.currentYear;
			iMonth = oCalendar.currentMonth - 1;
		}
		oCalendar.setMonth(iYear, iMonth);
	}
	
	//==========================================================
	//
	//	Function	:	clickNextMonth
	//
	//	Summary		:	Event handler of click a next month
	//
	//==========================================================
	function clickNextMonth(oButton)
	{
		let oCalendar = null;
		let iYear;
		let iMonth;
		
		oCalendar = oButton.parent;
		if(oCalendar.currentMonth == 11)
		{
			iYear = oCalendar.currentYear + 1;
			iMonth = 0;
		}
		else
		{
			iYear = oCalendar.currentYear;
			iMonth = oCalendar.currentMonth + 1;
		}
		oCalendar.setMonth(iYear, iMonth);
	}
	
}(jaswin.calendar = jaswin.calendar || {}));
