//==================================================================
//
//	File name	:	checkbox.js
//
//	Summary		:	CheckBox, radio, toggle classes
//
//	Ceated		:	2018/11/24
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (checkbox) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	checkbox.STYLE_DEFAULT = 0;
	checkbox.STYLE_BUTTON = 1;
	checkbox.STYLE_FLAT_BUTTON = 2;
	checkbox.STYLE_IMAGE = 4;
	checkbox.STYLE_ICON = 8;
	
	
	//==========================================================
	//
	//	Class		:	CheckBox
	//
	//	Summary		:	Class of checkbox
	//
	//==========================================================
	checkbox.CheckBox = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// * ID[string]
					oRect,			// * Size and position[FixedRect]
					iStyle,			// * Item style[int]
					bEnabled,		// * Enabled or not[boolean]
					bChecked,		// * Checked or not[boolean]
					sGroup,			// Group for radio button[String]
					sContents,		// Label text or image URL or icon name[string]
					sTooltip,		// Tool tip text[String]
					iTabIndex,		// Tab index[int]
					fpChange)		// Function on change
		{
			// Base class
			super(false, sID, false, oRect, bEnabled, iTabIndex);
			
			// Check parameters
			if(typeof iStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			
			if(typeof bChecked != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CHECKED, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			
			if(sGroup)
			{
				if(typeof sGroup != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_GROUP, sID),
												"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			}
			
			if(sContents)
			{
				if(typeof sContents != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, sID),
												"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			}
			
			if(sTooltip)
			{
				if(typeof sTooltip != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
												"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			}
			
			if(fpChange)
			{
				if(typeof fpChange != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
												"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
			}
			
			// Set properties
			this.style = iStyle;			// Checkbox style[int]
			this.group = null;				// Group for radio button[string]
			this.checked = bChecked;		// Checked or not[boolean]
			this.onChange = fpChange;		// Function on change[function(Argument:CheckBox class)]
			this.box = null;				// Box for checkbox or radio button[DOM Element]
			this.content = null;			// Content element for text or image[DOM Element]
			
			if((sGroup) && (sGroup != ""))
				this.group = sGroup;
			
			// Create element
			this.element = document.createElement("div");
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			this.focusElement = this.element;
			
			if((this.style & checkbox.STYLE_BUTTON)
			|| (this.style & checkbox.STYLE_FLAT_BUTTON))
			{
				// Toggle button
				if(this.enabled)
				{
					if(this.checked)
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click";
						else
							this.element.className = "button-click";
					}
					else
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
					}
				}
				else
				{
					if(this.checked)
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click-disabled";
						else
							this.element.className = "button-click-disabled";
						this.element.style.cursor = "default";
					}
					else
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
						this.element.style.cursor = "default";
					}
				}
				
				this.element.style.width = this.rect.width - 2 + "px";		// 2:border
				this.element.style.height = this.rect.height - 2 + "px";	// 2:border
			}
			else
			{
				// CheckBox or Radio button
				this.element.className = "checkbox";
				this.element.style.width = this.rect.width + "px";
				this.element.style.height = this.rect.height + "px";
				
				if(!this.enabled)
					this.element.style.cursor = "default";
			}
			
			// Create contents
			if((this.style & checkbox.STYLE_BUTTON)
			|| (this.style & checkbox.STYLE_FLAT_BUTTON))
			{
				// Toggle button
				if(this.style & checkbox.STYLE_IMAGE)
				{
					// Image button
					this.content = document.createElement("img");
					if(this.enabled)
					{
						if(this.checked)
							this.content.className = "button-image-click";
						else
							this.content.className = "button-image";
					}
					else
					{
						if(this.checked)
							this.content.className = "button-image-click-disabled";
						else
							this.content.className = "button-image-disabled";
					}
					this.content.src = sContents;
				}
				else if(this.style & checkbox.STYLE_ICON)
				{
					// Icon button
					this.content = document.createElement("span");
					if(this.enabled)
					{
						if(this.checked)
							this.content.className = "button-icon-click";
						else
							this.content.className = "button-icon";
					}
					else
					{
						if(this.checked)
							this.content.className = "button-icon-click-disabled";
						else
							this.content.className = "button-icon-disabled";
					}
					
					this.content.style.width = this.rect.width - 2 - 2 + "px";		// 2:border 2:margin
					this.content.style.height = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
					
					let oIconSpan = document.createElement("span");
					oIconSpan.className = sContents;
					this.content.appendChild(oIconSpan);
				}
				else if((sContents) && (sContents.length > 0))
				{
					// Text label
					this.content = document.createElement("span");
					if(this.enabled)
					{
						if(this.checked)
							this.content.className = "button-text-click";
						else
							this.content.className = "button-text";
					}
					else
					{
						if(this.checked)
							this.content.className = "button-text-click-disabled";
						else
							this.content.className = "button-text-disabled";
					}
					
					this.content.style.lineHeight = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
					this.content.innerHTML = sContents;
				}
				this.element.appendChild(this.content);
			}
			else
			{
				if(this.group)
				{
					// Radio button
					this.box = document.createElement("span");
					if(this.enabled)
					{
						if(this.checked)
							this.box.className = jaswin.ICON_RADIO_CHECKED;
						else
							this.box.className = jaswin.ICON_RADIO;
					}
					else
					{
						if(this.checked)
							this.box.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
						else
							this.box.className = jaswin.ICON_RADIO;
					}
					
					this.box.style.left = "2px";
					this.box.style.top = "5px";
				}
				else
				{
					// Check box
					this.box = document.createElement("span");
					if(this.enabled)
					{
						if(this.checked)
							this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
						else
							this.box.className = jaswin.ICON_CHECKBOX;
					}
					else
					{
						if(this.checked)
							this.box.className = ICON_CHECKBOX_CHECKED_DISABLED;
						else
							this.box.className = jaswin.ICON_CHECKBOX;
					}
					
					this.box.style.left = "1px";
					this.box.style.top = "4px";
				}
				this.box.style.postion = "absolute";
				this.element.appendChild(this.box);
				this.content = document.createElement("span");
				if(this.enabled)
					this.content.className = "checkbox-label";
				else
				{
					this.content.className = "checkbox-label-disabled";
					this.box.style.cursor = "default";
				}
				this.content.style.left = "16px";
				this.content.style.top = "0px";
				this.content.style.width = this.rect.width - 16 + "px";
				this.content.style.height = this.rect.height + "px";
				
				this.content.innerHTML = sContents;
				this.element.appendChild(this.content);
			}
			
			if((sTooltip) && (sTooltip.length > 0))
				this.element.title = sTooltip;
			
			// Tab index
			if(iTabIndex)
			{
				if(iTabIndex > 0)
				{
					this.tabIndex = iTabIndex;
					this.focusElement.tabIndex = this.tabIndex;
				}
			}
			
			if(this.enabled)
			{
				this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
					this.setHover();
				}, false);
				this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
					this.setOut();
				}, false);
				this.focusElement.addEventListener("focus", this.focusEvent = (e) => {
					if((!this.enabled) || (!this.parentEnabled))
					{
						e.preventDefault();
						return;
					}
					this.setHover();
				}, false);
				this.focusElement.addEventListener("blur", this.blurEvent = (e) => {
					if((!this.enabled) || (!this.parentEnabled))
					{
						e.preventDefault();
						return;
					}
					this.setOut();
				}, false);
				this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
					this.changeStatus();
				}, false);
				this.element.addEventListener("keyup", this.keyUpEvent = (e) => {
					if(e.keyCode != 32)
						return true;
					this.changeStatus();
				}, false);
			}
			
		}
		
		//==================================================
		//	Enable this checkbox
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if(this.enabled)
			{
				if((this.style & checkbox.STYLE_BUTTON)
				|| (this.style & checkbox.STYLE_FLAT_BUTTON))
				{
					if(this.checked)
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click";
						else
							this.element.className = "button-click";
						
						if(this.content)
						{
							if(this.style & checkbox.STYLE_IMAGE)
								this.content.className = "button-image-click";
							else if(this.style & checkbox.STYLE_ICON)
								this.content.className = "button-icon-click";
							else
								this.content.className = "button-text-click";
						}
					}
					else
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
						
						if(this.content)
						{
							if(this.style & checkbox.STYLE_IMAGE)
								this.content.className = "button-image";
							else if(this.style & checkbox.STYLE_ICON)
								this.content.className = "button-icon";
							else
								this.content.className = "button-text";
						}
					}
				}
				else
				{
					if(this.group)
					{
						if(this.checked)
							this.box.className = jaswin.ICON_RADIO_CHECKED;
						else
							this.box.className = jaswin.ICON_RADIO;
					}
					else
					{
						if(this.checked)
							this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
						else
							this.box.className = jaswin.ICON_CHECKBOX;
					}
					this.box.style.cursor = "pointer";
					
					if(this.content)
						this.content.className = "checkbox-label";
				}
				
				this.element.style.cursor = "pointer";
				
				if(!this.mouseMoveEvent)
				{
					this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
						this.setHover();
					}, false);
				}
				if(!this.mouseOutEvent)
				{
					this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
						this.setOut();
					}, false);
				}
				if(!this.focusEvent)
				{
					this.focusElement.addEventListener("focus", this.focusEvent = (e) => {
						if((!this.enabled) || (!this.parentEnabled))
						{
							e.preventDefault();
							return;
						}
						this.setHover();
					}, false);
				}
				if(!this.blurEvent)
				{
					this.focusElement.addEventListener("blur", this.blurEvent = (e) => {
						if((!this.enabled) || (!this.parentEnabled))
						{
							e.preventDefault();
							return;
						}
						this.setOut();
					}, false);
				}
				if(!this.mouseDownEvent)
				{
					this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
						this.changeStatus();
					}, false);
				}
				if(!this.keyUpEvent)
				{
					this.element.addEventListener("keyup", this.keyUpEvent = (e) => {
						if(e.keyCode != 32)
							return true;
						this.changeStatus();
					}, false);
				}
			}
		}
		
		//==================================================
		//	Disable this checkbox
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			if((this.style & checkbox.STYLE_BUTTON)
			|| (this.style & checkbox.STYLE_FLAT_BUTTON))
			{
				if(this.checked)
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat-click-disabled";
					else
						this.element.className = "button-click-disabled";
					
					if(this.content)
					{
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-click-disabled";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-click-disabled";
						else
							this.content.className = "button-text-click-disabled";
					}
				}
				else
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat";
					else
						this.element.className = "button";
					
					if(this.content)
					{
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-disabled";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-disabled";
						else
							this.content.className = "button-text-disabled";
					}
				}
			}
			else
			{
				if(this.content)
					this.content.className = "checkbox-label-disabled";
				if(this.group)
				{
					if(this.checked)
						this.box.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_RADIO;
				}
				else
				{
					if(this.checked)
						this.box.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				
				this.box.style.cursor = "default";
			}
			this.element.style.cursor = "default";
			
			if(this.mouseMoveEvent)
			{
				this.element.removeEventListener("mousemove", this.mouseMoveEvent, false);
				this.mouseMoveEvent = null;
			}
			if(this.mouseOutEvent)
			{
				this.element.removeEventListener("mouseout", this.mouseOutEvent, false);
				this.mouseOutEvent = null;
			}
			if(this.focusEvent)
			{
				this.focusElement.removeEventListener("focus", this.focusEvent, false);
				this.focusEvent = null;
			}
			if(this.blurEvent)
			{
				this.focusElement.removeEventListener("blur", this.blurEvent, false);
				this.blurEvent = null;
			}
			if(this.mouseDownEvent)
			{
				this.element.removeEventListener("mousedown", this.mouseDownEvent, false);
				this.mouseDownEvent = null;
			}
			if(this.keyUpEvent)
			{
				this.element.removeEventListener("keyup", this.keyUpEvent, false);
				this.keyUpEvent = null;
			}
		}
		
		//==================================================
		//	Set this checkbox hover
		//==================================================
		setHover()
		{
			if(this.checked)
				return;
			
			if(this.style & checkbox.STYLE_FLAT_BUTTON)
				this.element.className = "button-flat-focus";
			else if(this.style & checkbox.STYLE_BUTTON)
				this.element.className = "button-focus";
			else
			{
				if(this.group)
					this.box.className = "icon-radio-hover";
				else
					this.box.className = "icon-check-hover";
			}
		}
		
		//==================================================
		//	Set this checkbox mouse out
		//==================================================
		setOut()
		{
			if(this.checked)
				return;
			
			if(this.style & checkbox.STYLE_FLAT_BUTTON)
				this.element.className = "button-flat";
			else if(this.style & checkbox.STYLE_BUTTON)
				this.element.className = "button";
			else
			{
				if(this.group)
					this.box.className = jaswin.ICON_RADIO;
				else
					this.box.className = jaswin.ICON_CHECKBOX;
			}
		}
		
		//==================================================
		//	Change status of this checkbox
		//==================================================
		changeStatus()
		{
			if(this.group)
			{
				// Radio button
				if(this.checked)
					return;
				
				let oaRadios = getGroupRadioButtons(this.parent, this.group);
				let oRadio;
				let iCounter;
				
				for(iCounter = 0; iCounter < oaRadios.length; iCounter++)
				{
					oRadio = oaRadios[iCounter];
					if((oRadio.group) && (oRadio.group == this.group))
					{
						// Only for same group
						if(oRadio == this)
						{
							this.checked = true;
							if((this.style & checkbox.STYLE_BUTTON)
							|| (this.style & checkbox.STYLE_FLAT_BUTTON))
							{
								if(this.style & checkbox.STYLE_FLAT_BUTTON)
									this.element.className = "button-flat-click";
								else
									this.element.className = "button-click";
								
								if(this.style & checkbox.STYLE_IMAGE)
									this.content.className = "button-image-click";
								else if(this.style & checkbox.STYLE_ICON)
									this.content.className = "button-icon-click";
								else
									this.content.className = "button-text-click";
							}
							else
								this.box.className = jaswin.ICON_RADIO_CHECKED;
						}
						else
						{
							oRadio.checked = false;
							if((this.style & checkbox.STYLE_BUTTON)
							|| (this.style & checkbox.STYLE_FLAT_BUTTON))
							{
								if(this.style & checkbox.STYLE_FLAT_BUTTON)
									oRadio.element.className = "button-flat";
								else
									oRadio.element.className = "button";
								
								if(this.style & checkbox.STYLE_IMAGE)
									oRadio.content.className = "button-image";
								else if(this.style & checkbox.STYLE_ICON)
									oRadio.content.className = "button-icon";
								else
									oRadio.content.className = "button-text";
							}
							else
								oRadio.box.className = jaswin.ICON_RADIO;
						}
					}
				}
			}
			else
			{
				// Checkbox
				if(this.checked)
				{
					this.checked = false;
					if((this.style & checkbox.STYLE_BUTTON)
					|| (this.style & checkbox.STYLE_FLAT_BUTTON))
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
						
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon";
						else
							this.content.className = "button-text";
					}
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				else
				{
					this.checked = true;
					if((this.style & checkbox.STYLE_BUTTON)
					|| (this.style & checkbox.STYLE_FLAT_BUTTON))
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click";
						else
							this.element.className = "button-click";
						
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-click";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-click";
						else
							this.content.className = "button-text-click";
					}
					else
						this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
				}
			}
			
			if(this.onChange)
				this.onChange(this);
		}
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel(sText)
		{
			if(!this.content)
				return;
			
			if((this.style & checkbox.STYLE_IMAGE)
			|| (this.style & checkbox.STYLE_ICON))
				return;
			
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setLabel");
			
			this.content.innerHTML = sText;
		}
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setTooltip");
			this.element.title = sTooltip;
		}
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setChangeEvent");
			
			this.onChange = fpChange;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getGroupRadioButtons
	//
	//	Summary		:	Get radio buttons of the same group
	//
	//==========================================================
	function getGroupRadioButtons(oContainer, sGroupName)
	{
		let iCounter;
		let oControl;
		let oaRadioButtons = new Array();
		
		for(iCounter = 0; iCounter < oContainer.controls.length; iCounter++)
		{
			if(!oContainer.controls[iCounter] instanceof jaswin.checkbox.CheckBox)
				continue;
			if((!oContainer.controls[iCounter].group)
			|| (oContainer.controls[iCounter].group != sGroupName))
				continue;
			oaRadioButtons.push(oContainer.controls[iCounter]);
		}
		
		return oaRadioButtons;
	}
	
}(jaswin.checkbox = jaswin.checkbox || {}));
